use crate::data::{Config, Mapping};
use crate::image::Image;

pub struct Pattern {
    pub image: Image,
    pub config: Config,
    pub mapping: Mapping,
}

impl Pattern {
    pub fn draw(&self, ctx: cairo::Context) -> Result<(), Box<dyn std::error::Error>> {
        ctx.set_source_rgb(1.0, 1.0, 1.0);
        ctx.paint()?;

        let layout = pangocairo::functions::create_layout(&ctx).unwrap();
        let font = pango::FontDescription::from_string(&self.config.font);
        layout.set_width(2880);
        layout.set_font_description(Some(&font));

        ctx.set_source_rgb(0.0, 0.0, 0.0);
        for i in 0..=self.image.width {
            ctx.move_to(self.config.scale(i), 0.0);
            ctx.line_to(self.config.scale(i), self.config.scale(self.image.height));
            ctx.set_line_width(if i % self.config.grid_every == 0 {
                self.config.major_line_weight
            } else {
                self.config.line_weight
            });
            ctx.stroke()?;
        }
        for j in 0..=self.image.height {
            ctx.move_to(0.0, self.config.scale(j));
            ctx.line_to(self.config.scale(self.image.width), self.config.scale(j));
            ctx.set_line_width(if j % self.config.grid_every == 0 {
                self.config.major_line_weight
            } else {
                self.config.line_weight
            });
            ctx.stroke()?;
        }

        ctx.set_source_rgb(0.0, 0.0, 0.0);

        for ((x, y), pixel) in self.image.iter() {
            let l = self.mapping.lookup(pixel);

            if !l.is_empty() {
                ctx.move_to(self.config.scale(x) + 5.0, self.config.scale(y) + 3.0);
                layout.set_text(l);
                pangocairo::functions::show_layout(&ctx, &layout);
            }
        }

        Ok(())
    }
}
